require "active_support"
require "active_support/core_ext"
require "badgerfish"

class Mab2::Document
  require_relative "./document/scope"

  attr_accessor :legacy_mabmapper_mode

  def initialize(xml, options = {})
    @badgerfish_hash = Badgerfish::Parser.new.load(xml, options)

    # the original mabmapper dsl is buggy at certain points, but this
    # behaviour might be needed for certain legacy rules
    @legacy_mabmapper_mode =
    if options[:legacy_mabmapper_mode].nil?
      false
    else
      !!options[:legacy_mabmapper_mode]
    end

    create_controlfields_tag_index
    create_datafields_tag_index
  end

  #
  # dsl methods
  #
  def controlfield(tag)
    controlfields_by_tag(tag)
    .first
    .try do |controlfield|
      controlfield["$"].chars.map! do |char|
        char == "|" ? nil : char
      end
    end || []
  end
 
  def doc
    self
  end

  def field(tag, options = {})
    Scope.new(options.merge(document: self, tag: tag))
  end

  #
  # other public methods
  #
  def controlfields_by_tag(tag)
    (@controlfields_tag_index[tag] || []).map do |index|
      controlfields[index]
    end
  end

  def datafields_by_tag(tag)
    (@datafields_tag_index[tag] || []).map do |index|
      datafields[index]
    end
  end

  def legacy_mabmapper_mode?
    @legacy_mabmapper_mode
  end

  #
  private
  #
  def create_controlfields_tag_index
    controlfields.each.with_index.inject(@controlfields_tag_index = {}) do |index, (controlfield, index_within_array)|
      index.tap do |_index|
        (_index[controlfield["@tag"]] ||= []).push(index_within_array)
      end
    end
  end

  def create_datafields_tag_index
    datafields.each.with_index.inject(@datafields_tag_index = {}) do |index, (datafield, index_within_array)|
      index.tap do |_index|
        (_index[datafield["@tag"]] ||= []).push(index_within_array)
      end
    end
  end

  def controlfields
    @controlfields ||=
    begin
      @badgerfish_hash["OAI-PMH"]["ListRecords"]["record"]["metadata"]["record"]["controlfield"].try do |controlfields|
        controlfields.is_a?(Array) ? controlfields : [controlfields]
      end || []
    end
  end

  def datafields
    @datafields ||=
    begin
      @badgerfish_hash["OAI-PMH"]["ListRecords"]["record"]["metadata"]["record"]["datafield"].try do |datafields|
        datafields.is_a?(Array) ? datafields : [datafields]
      end || []
    end
  end
end
