require "active_support"
require "active_support/core_ext"
require "mab2/document"

class Mab2::Document::Scope
  class Subfield < Struct.new(:code, :value)
    alias_method :name, :code
  end

  def initialize(options = {})
    @document = options[:document]
    @ind1 = [options[:ind1]].flatten(1).compact
    @ind2 = [options[:ind2]].flatten(1).compact
    @tag = options[:tag]
    @subfield = [options[:subfield]].flatten(1).compact

    # negation extraction
    @ind1.delete_if do |ind1|
      if ind1.start_with?("-") && ind1.length > 1
        (@not_ind1 ||= []).push(ind1[1..-1])
        true
      end
    end
  end

  def get
    self
  end

  def get_subfield(code)
    [datafields.first["subfield"]].compact.flatten(1)
    .try(:find) do |subfield|
      subfield["@code"] == code
    end
    .try do |subfield|
      Subfield.new(subfield["@code"], subfield["$"])
    end || Subfield.new
  end

  def fields
    datafields.map do |datafield|
      self.class.new({
        document: @document,
        ind1: datafield["@ind1"],
        ind2: datafield["@ind2"],
        tag: datafield["@tag"]
      })
    end
  end

  def subfield(codes)
    self.class.new(to_options.merge(subfield: codes))
  end

  def subfields
    datafields.map do |datafield|
      [datafield["subfield"]].compact.flatten(1).try(:map) do |subfield|
        Subfield.new(subfield["@code"], subfield["$"])
      end
    end.compact.flatten(1) || []
  end

  def value(options = {})
    separator = options[:join_subfields] || " "

    if @document.legacy_mabmapper_mode?
      values.first.join(separator)
    else
      values.map(&:presence).compact.first.try(:join, separator)
    end
  end

  def values(options = {})
    datafields
    .map do |field|
      [field["subfield"]].compact.flatten(1)
      .select do |subfield|
        @subfield.empty? || @subfield.include?(subfield["@code"])
      end
      .map do |subfield|
        subfield["$"]
      end
      .try do |subfield_values|
        if separator = options[:join_subfields]
          subfield_values.join(separator)
        else
          subfield_values
        end
      end
    end
  end

  #
  private
  #
  def datafields
    @document.datafields_by_tag(@tag)
    .dup
    .tap do |fields|
      fields.select! do |field|
        (@ind1.empty? || @ind1.include?(field["@ind1"]) || (@not_ind1.present? && !@not_ind1.include?(field["@ind1"]))) &&
        (@ind2.empty? || @ind2.include?(field["@ind2"]))
      end
    end
  end

  def to_options
    {
      document: @document,
      ind1: @ind1,
      ind2: @ind2,
      tag: @tag,
      subfield: @subfield
    }
  end
end
